@@ -18,7 +18,7 @@
     License along with this library; if not, write to the Free
     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: util.c 568 2006-07-24 10:19:58Z kmaraas $
+    $Id: util.c 600 2009-01-22 12:30:07Z kmaraas $
 
 ***************************************************************************/
 #include <assert.h>
@@ -37,9 +37,12 @@
 
 #ifdef G_OS_WIN32
 #include <fcntl.h>
+#include <direct.h>
+#define popen _popen
+#define pclose _pclose
 #endif
 
-IDL_EXPORT const char *IDL_tree_type_names[] = {
+const char *IDL_tree_type_names[] = {
 	"IDLN_NONE",
 	"IDLN_ANY",
 	"IDLN_LIST",
@@ -88,9 +91,9 @@ IDL_EXPORT const char *IDL_tree_type_names[] = {
 	/* IDLN_LAST */
 };
 
-IDL_EXPORT int				__IDL_check_type_casts = FALSE;
+int					__IDL_check_type_casts = FALSE;
 #ifndef HAVE_CPP_PIPE_STDIN
-char *					__IDL_tmp_filename = NULL;
+const char *				__IDL_tmp_filename = NULL;
 #endif
 const char *				__IDL_real_filename = NULL;
 char *					__IDL_cur_filename = NULL;
@@ -224,13 +227,17 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	FILE *input;
 	char *cmd;
 #ifdef HAVE_CPP_PIPE_STDIN
-	char *fmt = CPP_PROGRAM " " CPP_NOSTDINC " - %s%s %s < \"%s\" %s";
+	char *fmt = CPP_PROGRAM " - %s%s %s < \"%s\" %s";
 	char *wd;
 #else
-	char *fmt = CPP_PROGRAM " " CPP_NOSTDINC " -I- -I%s %s \"%s\" %s";
-	char *s, *tmpfilename;
+	char *fmt = CPP_PROGRAM " -I%s %s \"%s\" %s";
 	char cwd[2048];
+#ifdef HAVE_SYMLINK
+	char *s, *tmpfilename;
 	gchar *linkto;
+#else
+	const char *tmpfilename;
+#endif
 #endif
 
 #ifndef G_OS_WIN32
@@ -267,11 +274,12 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 
 	g_free (wd);
 #else
-	s = tmpnam (NULL);
-	if (s == NULL)
+	if (!getcwd (cwd, sizeof (cwd)))
 		return -1;
 
-	if (!getcwd (cwd, sizeof (cwd)))
+#ifdef HAVE_SYMLINK
+	s = tmpnam (NULL);
+	if (s == NULL)
 		return -1;
 
 	if (g_path_is_absolute (filename)) {
@@ -281,16 +289,16 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	}
 
 	tmpfilename = g_strconcat (s, ".c", NULL);
-#ifdef HAVE_SYMLINK
+
 	if (symlink (linkto, tmpfilename) < 0) {
 		g_free (linkto);
 		g_free (tmpfilename);
 		return -1;
 	}
-#else
-#error Must have symlink
-#endif
 	g_free (linkto);
+#else
+	tmpfilename = filename;
+#endif
 
 	cmd = g_strdup_printf (fmt, cwd, cpp_args ? cpp_args : "",
 			       tmpfilename, cpperrs);
@@ -301,7 +309,7 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	putenv ("LC_ALL=C");
 
 #ifdef HAVE_POPEN
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_MSC_VER)
 	if (!(parse_flags & IDLF_SHOW_CPP_ERRORS)) {
 		int save_stderr = dup (2);
 		int null = open ("NUL:", O_WRONLY);
@@ -317,13 +325,12 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 	input = popen (cmd, "r");
 #endif
 #else
-	/* Surely this doesn't make any sense? */	
-	input = fopen (cmd, "r");
+#error Must have popen
 #endif
 	g_free (cmd);
 
 	if (input == NULL || ferror (input)) {
-#ifndef HAVE_CPP_PIPE_STDIN
+#if !defined (HAVE_CPP_PIPE_STDIN) && defined (HAVE_SYMLINK)
 		g_free (tmpfilename);
 #endif
 		return IDL_ERROR;
@@ -356,7 +363,7 @@ int IDL_parse_filename (const char *filename, const char *cpp_args,
 #else
 	fclose (input);
 #endif
-#ifndef HAVE_CPP_PIPE_STDIN
+#if !defined (HAVE_CPP_PIPE_STDIN) && defined (HAVE_SYMLINK)
 	unlink (tmpfilename);
 	g_free (tmpfilename);
 #endif
@@ -502,9 +509,10 @@ void yyerrorl (const char *s, int ofs)
 	__IDL_is_okay = FALSE;
 
 	/* Errors are counted, even if not printed */
-	if (__IDL_max_msg_level < IDL_ERROR)
+	if (__IDL_max_msg_level < IDL_ERROR) {
 		g_free (freeme);
 		return;
+	}
 
 	if (__IDL_msgcb)
 		(*__IDL_msgcb)(IDL_ERROR, __IDL_nerrors, line, filename, s);
@@ -2596,7 +2604,7 @@ static gboolean IDL_output_delim_pre (IDL_tree_func_data *tfd, IDL_output_delim_
 {
 	if (IDL_output_delim_match (tfd, delim)) {
 		if (delim->hit)
-			dataf (delim->data, delim->delim);
+			dataf (delim->data, "%s", delim->delim);
 		else
 			delim->hit = TRUE;
 		return delim->pre_func
@@ -2762,7 +2770,7 @@ static gboolean IDL_emit_IDL_ident_real (IDL_tree_func_data *tfd, IDL_output_dat
 		   prepending an underscore. */
 		dataf (data, "%s", IDL_IDENT (tfd->tree).str);
 	} else {
-		if ( up_path == 0 ) {
+		if (up_path == NULL) {
 		    levels = 0;
 		} else {
 		    /* Determine minimal required levels of scoping */
@@ -3520,7 +3528,7 @@ static gboolean
 contains_node_walker (IDL_tree_func_data *tfd, gpointer data) {
 	ContainsNodeWalkerInfo	*info = data;
 	/* skip root! */
-	if ( tfd->up!=0 && tfd->tree == info->searchNode ) {
+	if (tfd->up != NULL && tfd->tree == info->searchNode) {
 		info->found = TRUE;
 		return FALSE;
 	}
